#include<iostream>
#include<vector>
#include<tuple>
#include<algorithm>
#include<stack>
#include<queue>
using namespace std;

const int MAXN=1e5, MAXM=1e6;

struct node
{
    int a, b, realIdx;
    bool visited;
};

//MST
vector<pair<int, int> > mstGraph[MAXN];
int adjCnt[MAXN], root[MAXN], prv[MAXM];
bool usedNode[MAXM];

vector<node> nodes;
vector<pair<int, int> > graph[MAXN], secondGraph[MAXN];

int GetRoot(int idx)
{
    if(root[idx]==idx)
    {
        return idx;
    }

    return GetRoot(root[idx]);
}

void GetMST(int n)
{
    for(int edge=0; edge<n; edge++)
    {
        for(int node=0; node<secondGraph[edge].size(); node++)
        {
            int target, nodeIdx;
            tie(target, nodeIdx)=secondGraph[edge][node];

            int r1=GetRoot(edge);
            int r2=GetRoot(target);

            if(r1!=r2)
            {
                root[r1]=r2;
                mstGraph[edge].push_back({target, nodeIdx});
                mstGraph[target].push_back({edge, nodeIdx});
            }
        }
    }

    return;
}

bool PairOdd(int source)
{
    queue<int> q;
    q.push(source);

    while(!q.empty())
    {
        int edge=q.front();
        q.pop();

        if(edge!=source && graph[edge].size()%2!=0)
        {
            while(edge!=source)
            {
                int nodeIdx=prv[edge];
                int prevEdge=(nodes[nodeIdx].b==edge?nodes[nodeIdx].a:nodes[nodeIdx].b);
                nodes.push_back({edge, prevEdge, nodeIdx, false});
                graph[edge].push_back({prevEdge, nodes.size()-1});
                graph[prevEdge].push_back({edge, nodes.size()-1});
                edge=prevEdge;
            }

            return true;
        }

        for(int i=0; i<mstGraph[edge].size(); i++)
        {
            int target, nodeIdx;
            tie(target, nodeIdx)=mstGraph[edge][i];

            if(usedNode[nodeIdx])
            {
                continue;
            }

            prv[target]=nodeIdx;
            usedNode[nodeIdx]=true;
            q.push(target);
        }
    }

    return false;
}

bool CutLeaves(int n)
{
    int oddCnt=0;
    for(int i=0; i<n; i++)
    {
        if(graph[i].size()!=0)
        {
            oddCnt++;
        }
    }

    if(oddCnt%2!=0)
    {
        return false;
    }

    vector<int> leaves;
    for(int edge=0; edge<n; edge++)
    {
        adjCnt[edge]=mstGraph[edge].size();
        if(adjCnt[edge]==1)
        {
            leaves.push_back(edge);
        }
    }

    for(int leaf=0; leaf<leaves.size(); leaf++)
    {
        if(graph[leaf].size()%2==0)
        {
            int parent=mstGraph[leaf][0].first;
            adjCnt[parent]--;

            if(adjCnt[parent]==1)
            {
                leaves.push_back(leaf);
            }
        }
        else
        {
            if(!PairOdd(leaf))
            {
                return false;
            }
        }
    }

    oddCnt=0;
    for(int i=0; i<n; i++)
    {
        if(graph[i].size()!=0)
        {
            oddCnt++;
        }
    }

    return oddCnt!=0;
}

void ClearMST(int n, int m)
{
    for(int i=0; i<n; i++)
    {
        adjCnt[i]=0;
        root[i]=i;
    }

    for(int i=0; i<m; i++)
    {
        prv[i]=-1;
        usedNode[i]=false;
    }
}

int main()
{
    //ios_base::sync_with_stdio(0);
    //cin.tie(0);

    int n, m;
    cin >> n >> m;

    ClearMST(n, m);
    for(int i=0; i<m; i++)
    {
        int a, b, t;
        cin >> a >> b >> t;
        a--, b--;

        nodes.push_back({a, b, i, false});
        graph[a].push_back({b, i});
        graph[b].push_back({a, i});

        if(t==2)
        {
            secondGraph[a].push_back({b, i});
            secondGraph[b].push_back({a, i});
        }
    }

    GetMST(n);

    if(!CutLeaves(n))
    {
        cout << "-1\n";

        return 0;
    }

    cout << "1\n";

    return 0;
}
